<?php
/**
 * Base class for Spec tests
 */
class SpecTestCase extends DrupalWebTestCase {
  /**
   * Run specs
   *
   * @param Array $scenarios scenarios to run
   */
  public function runSpecs(array $scenarios = array()) {
    // Initialize verbose debugging.
    simpletest_verbose(NULL, variable_get('file_public_path', conf_path() . '/files'), get_class($this));

    // HTTP auth settings (<username>:<password>) for the simpletest browser
    // when sending requests to the test site.
    $this->httpauth_method = variable_get('simpletest_httpauth_method', CURLAUTH_BASIC);
    $username = variable_get('simpletest_httpauth_username', NULL);
    $password = variable_get('simpletest_httpauth_password', NULL);
    if ($username && $password) {
      $this->httpauth_credentials = $username . ':' . $password;
    }

    set_error_handler(array($this, 'specErrorHandler'));
    $class = get_class($this);
    // Iterate through all the scenarios in this class, unless a specific list of
    // scenarios to run was passed.
    $class_methods = get_class_methods($class);
    if ($scenarios) {
      $class_methods = array_intersect($class_methods, $scenarios);
    }

    // Preserve session of the current user running the tests
    $this->saveOriginalSession();

    foreach ($class_methods as $method) {
      // If the current method starts with "test", run it - it's a test.
      if (strtolower(substr($method, 0, 4)) == 'test') {
        // Insert a fail record. This will be deleted on completion to ensure
        // that testing completed.
        $method_info = new ReflectionMethod($class, $method);
        $caller = array(
          'file' => $method_info->getFileName(),
          'line' => $method_info->getStartLine(),
          'function' => $class . '->' . $method . '()',
        );
        $completion_check_id = DrupalTestCase::insertAssert($this->testId, $class, FALSE, t('The test did not complete due to a fatal error.'), 'Completion check', $caller);
        $this->setUp();
        try {
          $this->$method();
          // Finish up.
        }
        catch (Exception $e) {
          $this->specExceptionHandler($e);
        }
        $this->tearDown();
        // Remove the completion check record.
        DrupalTestCase::deleteAssert($completion_check_id);
      }
    }

    // Restore session of the user running the tests
    $this->restoreOriginalSession();

    // Clear out the error messages and restore error handler.
    drupal_get_messages();
    restore_error_handler();
  }

  /**
   * Wrapper function for SimpleTest's original run() method.
   * Used by SimpleTest's admin tool and drush method.
   * @see run() in drupal_web_test_case.php
   */
  public function run(array $methods = array()) {
    // Preserve session of the current user running the tests
    $this->saveOriginalSession();

    // Run the tests
    parent::run($methods);

    // Restore session of the user running the tests
    $this->restoreOriginalSession();
  }

  /**
   * Preserve current session ID of the logged in user.
   * We might need to restore this session later.
   *
   * @see login() which changes the session of the currently logged
   * in user to the user specified. If in use, the session of the user who's
   * running the tests must be restored so SimpleTest can complete the batch
   * and display results in the browser.
   */
  protected function saveOriginalSession() {
    $this->originalSessionId = session_id();
  }

  /**
   * Restore session of the user who's running the tests
   * in case it got changed during test runtime.
   */
  protected function restoreOriginalSession() {
    if (session_id() != $this->originalSessionId) {
      $this->regenerateSession($this->originalSessionId);
    }
  }

  /**
   * Clone of drupal_session_regenerate().
   * Adds option to specify the session ID of the new session.
   * Used to restore the session of the original user who's running the tests,
   * in case it got changed to another logged in user during test runtime (e.g.
   * through login()).
   * @see saveOriginalSession(), restoreOriginalSession()
   * @see drupal_session_regenerate() in session.inc
   *
   * @param $session_id optional
   */
  protected function regenerateSession($session_id = NULL) {
    global $user, $is_https;
    if ($is_https && variable_get('https', FALSE)) {
      $insecure_session_name = substr(session_name(), 1);
      if (!isset($GLOBALS['lazy_session']) && isset($_COOKIE[$insecure_session_name])) {
        $old_insecure_session_id = $_COOKIE[$insecure_session_name];
      }
      $params = session_get_cookie_params();
      $session_id = drupal_hash_base64(uniqid(mt_rand(), TRUE) . drupal_random_bytes(55));
      // If a session cookie lifetime is set, the session will expire
      // $params['lifetime'] seconds from the current request. If it is not set,
      // it will expire when the browser is closed.
      $expire = $params['lifetime'] ? REQUEST_TIME + $params['lifetime'] : 0;
      setcookie($insecure_session_name, $session_id, $expire, $params['path'], $params['domain'], FALSE, $params['httponly']);
      $_COOKIE[$insecure_session_name] = $session_id;
    }

    if (drupal_session_started()) {
      $old_session_id = session_id();
    }

    // Set current session ID using a specified (previously used) ID in order to restore
    // an old session or generate a new ID.
    $session_id = isset($session_id) ? $session_id : drupal_hash_base64(uniqid(mt_rand(), TRUE) . drupal_random_bytes(55));
    session_id($session_id);

    if (isset($old_session_id)) {
      $params = session_get_cookie_params();
      $expire = $params['lifetime'] ? REQUEST_TIME + $params['lifetime'] : 0;
      setcookie(session_name(), session_id(), $expire, $params['path'], $params['domain'], $params['secure'], $params['httponly']);
      $fields = array('sid' => session_id());
      if ($is_https) {
        $fields['ssid'] = session_id();
        // If the "secure pages" setting is enabled, use the newly-created
        // insecure session identifier as the regenerated sid.
        if (variable_get('https', FALSE)) {
          $fields['sid'] = $session_id;
        }
      }
      db_update('sessions')
        ->fields($fields)
        ->condition($is_https ? 'ssid' : 'sid', $old_session_id)
        ->execute();
    }
    elseif (isset($old_insecure_session_id)) {
      // If logging in to the secure site, and there was no active session on the
      // secure site but a session was active on the insecure site, update the
      // insecure session with the new session identifiers.
      db_update('sessions')
        ->fields(array('sid' => $session_id, 'ssid' => session_id()))
        ->condition('sid', $old_insecure_session_id)
        ->execute();
    }
    else {
      // Start the session when it doesn't exist yet.
      // Preserve the logged in user, as it will be reset to anonymous
      // by _drupal_session_read.
      $account = $user;
      drupal_session_start();
      $user = $account;
    }
    date_default_timezone_set(drupal_get_user_timezone());
  }

  /**
   * Custom implementation of assert method.
   * Logs each assertion internally so the spec runner can tell
   * which step and which script has passed or failed.
   *
   * @param DrupalTestCase::assert()
   */
  protected function assert($status, $message = '', $group = 'Other', array $caller = NULL) {
    // Convert boolean status to string status.
    if (is_bool($status)) {
      $status = $status ? 'pass' : 'fail';
    }
    else if ($status == 'exception') {
      $status = 'fail';
    }

    $this->logStep($status, $message);
    return parent::assert($status, $message, $group, $caller);
  }

  /**
   * Find all Gherkin steps (Given, When, Then, And, But) in a scenario.
   *
   * @param String $class name of Test class
   * @param String $method name of scenario method
   * @return Array of scenario steps ordererd by line number
   */
  public function getSteps($class, $method) {
    $test_method = new ReflectionMethod($class, $method);
    $filename = $test_method->getFileName();
    $start_line = $test_method->getStartLine();
    $end_line = $test_method->getEndLine();
    $source = file($filename);

    $steps = array();
    foreach ($source as $i => $line) {
      $i++;
      if ($i >= $start_line && $i <= $end_line && substr(trim($line), 0, 2) == '//') {
        $line = trim(substr(trim($line), 2));
        if (preg_match('/^[GIVEN|WHEN|THEN|AND|BUT]+/i', $line)) {
          $steps[$i] = $line;
        }
      }
    }

    return $steps;
  }

  /**
   * Stores logs with context information
   */
  public $specLog = array();

  /**
   * Stores information about the last feature/scenario/step
   */
  protected $last = array();

  /**
   * Log a scenario step internally
   *
   * @param String $status value is 'pass' or 'fail'
   * @param String $message the log message
   */
  private function logStep($status, $message = '') {
    // Find script and scenario method
    $script_method = $this->findScriptMethod();

    if (!$script_method) {
      return;
    }

    $srm = $script_method['function'];
    $scenario_method = $sm = $script_method['parent_function'];
    $scenario_example_number = $sem = is_null($this->currentOutlineCounter) ? 0 : $this->currentOutlineCounter;
    $scenario_example =  array(
      'number' => $scenario_example_number,
      'params' => is_null($this->currentOutline) ? array() : $this->currentOutline,
    );

    // Find the previous step for this method
    $steps = $this->getSteps($script_method['parent_class'], $scenario_method);
    $step = $this->findStep($steps, $script_method['line']);

    // Parse the step text
    $step_text = $st = $this->parseStepText($step['text']);

    $log = array(
      'line' => $step['line'],
      'text' => $step_text,
      'message' => $message,
      'scenario_method' => $scenario_method,
      'scenario_example' => $scenario_example,
      'script' => $srm,
      'status' => $status,
    );

    // Store as multidimensional array:
    // scenarios -> examples -> steps -> scripts -> logs

    if (!isset($this->specLog[$sm])) {
      $this->specLog[$sm] = array(
        'status' => 'pass',
        'examples' => array(),
      );
    }

    if (!isset($this->specLog[$sm]['examples'][$sem])) {
      $this->specLog[$sm]['examples'][$sem] = array(
        'status' => 'pass',
        'steps' => array(),
      );
    }

    if (!isset($this->specLog[$sm]['examples'][$sem]['steps'][$st])) {
      $this->specLog[$sm]['examples'][$sem]['steps'][$st] = array(
        'status' => 'pass',
        'scripts' => array(),
      );
    }

    if (!isset($this->specLog[$sm]['examples'][$sem]['steps'][$st]['scripts'][$srm])) {
      $this->specLog[$sm]['examples'][$sem]['steps'][$st]['script'][$srm] = array(
        'status' => 'pass',
        'logs' => array(),
      );
    }

    $this->specLog[$sm]['examples'][$sem]['steps'][$st]['scripts'][$srm]['logs'][] = $log;

    if ($status == 'fail') {
      $this->specLog[$sm]['status'] =
      $this->specLog[$sm]['examples'][$sem]['status'] =
      $this->specLog[$sm]['examples'][$sem]['steps'][$st]['status'] =
      $this->specLog[$sm]['examples'][$sem]['steps'][$st]['scripts'][$srm]['status'] = 'fail';
    }

    // Realtime logging
    //
    // This implementation is not logging the execution of features/scenarios/steps/scripts
    // in realtime _while_ they are beinng processed, but _after_ they have been processed.
    //
    // This is not an ideal outcome, yet better than no live logging at all and certainly
    // better than having to wait for the full test run to finish (a la SimpleTest).
    //
    // To achieve realtime logging, a completely different implementation would be
    // necessary. It's required to know the spec strucutre (feature -> scenario
    // -> steps -> scripts) upfront, so it would be possible for the logger to understand
    // where in the specs structure it is at the moment of logging an event and decide what
    // to output.
    // Example: output the first feature description, scenario name and step BEFORE
    // the first script would log anything.
    // To make this happen, the realtime logging mechanism would need to be implemented
    // outside of logStep() as this method only gets called upon an actual logging event,
    // i.e. an assertion.
    if (isset($GLOBALS['realtime_logging']) && $GLOBALS['realtime_logging'] == TRUE) {
      // Log current scenario
      $scenario_string = _scenario_string($scenario_method);
      if (!isset($this->last['scenario']) || $scenario_string != $this->last['scenario']) {
        drush_print("\n". _spec_indent(1) . 'Scenario: '.$scenario_string);
        $this->last['scenario'] = $scenario_string;
      }

      // Log current step
      if (!isset($this->last['step']) || $step_text != $this->last['step']) {
        drush_print(_spec_indent(2) . $step_text);
        $this->last['step'] = $step_text;
      }

      // Log current script with 'passed' or 'failed'
      if ($log['status'] == 'fail') {
        drush_log("\033[31m". _spec_indent(3) . _spec_filter_log_message($message) ."\033[0m", 'failed');
      }
      else {
        drush_log("\033[32m". _spec_indent(3) . _spec_filter_log_message($message) ."\033[0m", 'success');
      }

      // Simulate slow tests
      // sleep(1);
    }
  }

  /**
   * Parse a scenario step and replace placeholders with
   * params from the current outline example.
   *
   * @param String $text scenario step text
   * @return String parsed scenario step text
   */
  private function parseStepText($text) {
    if (!is_null($this->currentOutline)) {
      // Make params available as variables for t() function
      $vars = array();
      foreach ($this->currentOutline as $name => $param) {
        // Only params that are strings
        if (!is_string($param)) {
          continue;
        }

        // @see format_string()
        $vars['!'. $name] = $vars['@'. $name] = $vars['%'. $name] = $param;
      }

      $text = t($text, $vars);
    }

    return $text;
  }

  /**
   * Find the current script method
   *
   * @return Array the script method
   */
  private function findScriptMethod() {
    $method = NULL;
    $backtrace = debug_backtrace();

    // Looping through the backtrace stack
    // (same for loop through nested exception backtrace stack)
    // 1) store the current method in the stack: $method = $_caller (at the end of the loop)
    // 2) if the next method in the stack is a scenario method, then the
    //    previously stored message must have been the script method we're
    //    looking for. Use it and get out of the loop.
    foreach ($backtrace as $i => $_caller) {
      // Look for the next scenario method in the stack (method name starts with 'test')
      if (strtolower(substr($_caller['function'], 0, 4)) == 'test') {
        $method['parent_class'] = $_caller['class'];
        $method['parent_function'] = $_caller['function'];
        break;
      }

      // Look for any exception that could might have gotten thrown
      else if ($_caller['function'] == 'specExceptionHandler' && isset($_caller['args'][0])) {
        $e_backtrace = $_caller['args'][0]->getTrace();
        foreach ($e_backtrace as $e_caller) {
          // Look for the next scenario method in the exception stack
          if (strtolower(substr($e_caller['function'], 0, 4)) == 'test') {
            $method['parent_class'] = $e_caller['class'];
            $method['parent_function'] = $e_caller['function'];

            // Found the script method that threw the exception.
            // Stop here.
            return $method;
          }

          // Store the current method. It might be the script method we're looking for.
          $method = $e_caller;
        }
      }

      // Store the current method. It might be the script method we're looking for.
      $method = $_caller;
    }

    if (!isset($method['parent_class'])) {
      return NULL;
    }

    return $method;
  }

  /**
   * Find a step by line number
   *
   * @param Array $steps
   * @param Integer $line line number
   * @return Array step text, line number
   */
  private function findStep($steps, $line) {
    for($i = $line; $i>=0; $i--) {
      if (isset($steps[$i])) {
        return array('text' => $steps[$i], 'line' => $i);
      }
    }

    return NULL;
  }

  private $currentOutlineArray = NULL;
  private $currentOutlineCounter = NULL;
  private $currentOutline = NULL;

  /**
   * Run the next example in a scenario outline until all
   * examples have been run.
   *
   *   Example:
   *
   *   while ($params = $this->outline(array(
   *     array('user' => 'max', 'color' => 'red'),
   *     array('user' => 'jim', 'color' => 'green'),
   *   ))) {
   *     // Given I am @user
   *     // Then my favorite color is @color
   *   }
   *
   * @param Array outline_array an ordered array of outline examples
   * @return Array current outline example
   */
  protected function outline($outline_array) {
    if (is_null($this->currentOutlineArray)) {
      $this->currentOutlineArray = $outline_array;
      $this->currentOutlineCounter = 1;
    }
    else if (count($this->currentOutlineArray) == 0) {
      // All examples have run so terminate the loop
      $this->currentOutlineArray = $this->currentOutlineCounter = $this->currentOutline = NULL;
      return FALSE;
    }

    $this->currentOutline = array_shift($this->currentOutlineArray);
    $this->currentOutlineCounter++;

    return $this->currentOutline;
  }

  /**
  * Handle errors during test runs.
  *
  * Because this is registered in set_error_handler(), it has to be public.
  * @see set_error_handler
  */
  public function specErrorHandler($severity, $message, $file = NULL, $line = NULL) {
    if ($severity & error_reporting()) {
      $error_map = array(
        E_STRICT => 'Run-time notice',
        E_WARNING => 'Warning',
        E_NOTICE => 'Notice',
        E_CORE_ERROR => 'Core error',
        E_CORE_WARNING => 'Core warning',
        E_USER_ERROR => 'User error',
        E_USER_WARNING => 'User warning',
        E_USER_NOTICE => 'User notice',
        E_RECOVERABLE_ERROR => 'Recoverable error',
      );

      $backtrace = debug_backtrace();
      $caller = _drupal_get_last_caller($backtrace);

      $message = 'Error: '. $message ." (in ". $caller['file'] .", line ". $caller['line'] .")";
      $this->error($message, $error_map[$severity], $caller);
    }
    return TRUE;
  }

  /**
   * Handle exceptions.
   *
   * @see set_exception_handler
   */
  protected function specExceptionHandler($exception) {
    $backtrace = $exception->getTrace();
    // Push on top of the backtrace the call that generated the exception.
    array_unshift($backtrace, array(
     'line' => $exception->getLine(),
     'file' => $exception->getFile(),
    ));
    require_once DRUPAL_ROOT . '/includes/errors.inc';

    $caller = _drupal_get_last_caller($backtrace);
    // The exception message is run through check_plain() by _drupal_decode_exception().
    $vars = _drupal_decode_exception($exception);
    $vars['!message'] = _spec_filter_log_message($vars['!message']);
    $message = t('%type: !message in %function (line %line of %file).', $vars);
    $this->error($message, 'Uncaught exception', $caller);
  }

  /**
   * SimpleTest Fixture methods
   *
   * Copied from simpletest_fixture_test_case.php
   */

  /**
   * Set up environment (with fixtures) to run a test
   */
  protected function setUpFixture() {
    // copied from DrupalWebTestCase::setup() and modified
    global $user, $language, $conf;

    // How this works:
    // - there needs to be a valid prefix like 'simpletest123456',
    // - the prefix will get passed along with the user agent header information
    // - Drupal will look for 'simpletest..' in the header and use it as db prefix
    //   for requests during testing (SimpleTest & Selenium)
    // => fixture db tables must all be prefixed too!

    // Retrieve the static prefix parameter stored in the fixture db info in settings.php
    $fixturedb_connection = Database::getConnectionInfo('fixture');
    $this->databasePrefix = $fixturedb_connection['default']['simpletest_prefix'];

    db_update('simpletest_test_id')
      ->fields(array('last_prefix' => $this->databasePrefix))
      ->condition('test_id', $this->testId)
      ->execute();

    // Prepare switch to fixture database
    Database::renameConnection('default', 'simpletest_original_default');
    foreach ($fixturedb_connection as $target => $value) {
      $fixturedb_connection[$target]['prefix'] = array(
        'default' => $value['prefix']['default'] . $this->databasePrefix,
      );
    }
    Database::addConnectionInfo('default', 'default', $fixturedb_connection['default']);

    // Store necessary current values before switching to the fixture database.
    $this->originalLanguage = $language;
    $this->originalLanguageDefault = variable_get('language_default');
    $this->originalFileDirectory = variable_get('file_public_path', conf_path() . '/files');
    $this->originalProfile = drupal_get_profile();
    $this->removeTables = variable_get('simpletest_remove_tables', TRUE);
    $clean_url_original = variable_get('clean_url', 0);

    // Save and clean shutdown callbacks array because it static cached and
    // will be changed by the test run. If we don't, then it will contain
    // callbacks from both environments. So testing environment will try
    // to call handlers from original environment.
    $callbacks = &drupal_register_shutdown_function();
    $this->originalShutdownCallbacks = $callbacks;
    $callbacks = array();

    // Create test directory ahead of installation so fatal errors and debug
    // information can be logged during installation process.
    // Use temporary files directory with the same prefix as the database.
    $public_files_directory  = $this->originalFileDirectory . '/simpletest/' . substr($this->databasePrefix, 10);
    $private_files_directory = $public_files_directory . '/private';
    $temp_files_directory    = $private_files_directory . '/temp';

    // Create the directories
    file_prepare_directory($public_files_directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);
    file_prepare_directory($private_files_directory, FILE_CREATE_DIRECTORY);
    file_prepare_directory($temp_files_directory, FILE_CREATE_DIRECTORY);
    $this->generatedTestFiles = FALSE;

    // Log fatal errors.
    ini_set('log_errors', 1);
    ini_set('error_log', $public_files_directory . '/error.log');

    // Reset all statics and variables to perform tests in a clean environment.
    $conf = array();
    drupal_static_reset();

    // Set the test information for use in other parts of Drupal.
    $test_info = &$GLOBALS['drupal_test_info'];
    $test_info['test_run_id'] = $this->databasePrefix;
    $test_info['in_child_site'] = FALSE;

    // $this->setUpInstall(func_get_args(), $public_files_directory, $private_files_directory, $temp_files_directory);

    // NB Caches get rebuilt when the fixture db is being generated
    // @see simpletest_fixture_generate() in simpletest_fixture.module

    // Register actions declared by any modules.
    actions_synchronize();

    // Reload global $conf array and permissions.
    $this->refreshVariables();
    $this->checkPermissions(array(), TRUE);

    // Reset statically cached schema for new database prefix.
    drupal_get_schema(NULL, TRUE);

    // Run cron once in that environment, as install.php does at the end of
    // the installation process.
    // TODO time-consuming. Do we need this?
    // drupal_cron_run();

    // Log in with a clean $user.
    $this->originalUser = $user;
    drupal_save_session(FALSE);
    $user = user_load(1);

    // TODO time-consuming. Do we need this?
    // $this->setUpVariables($clean_url_original);

    // Set up English language.
    unset($GLOBALS['conf']['language_default']);
    $language = language_default();

    // Use the test mail class instead of the default mail handler class.
    variable_set('mail_system', array('default-system' => 'TestingMailSystem'));

    drupal_set_time_limit($this->timeLimit);

    $this->setup = TRUE;
  }

  /**
   * Delete created files and temporary files directory
   * and reset the database prefix.
   */
  protected function tearDownFixture() {
    // copied from DrupalWebTestCase::tearDown() and modified

    global $user, $language;

    // In case a fatal error occured that was not in the test process read the
    // log to pick up any fatal errors.
    simpletest_log_read($this->testId, $this->databasePrefix, get_class($this), TRUE);

    $emailCount = count(variable_get('drupal_test_email_collector', array()));
    if ($emailCount) {
      $message = format_plural($emailCount, '1 e-mail was sent during this test.', '@count e-mails were sent during this test.');
      $this->pass($message, t('E-mail'));
    }

    // Delete temporary files directory.
    file_unmanaged_delete_recursive($this->originalFileDirectory . '/simpletest/' . substr($this->databasePrefix, 10));

    // Get back to the original connection.
    Database::removeConnection('default');
    Database::renameConnection('simpletest_original_default', 'default');

    // Restore original shutdown callbacks array to prevent original
    // environment of calling handlers from test run.
    $callbacks = &drupal_register_shutdown_function();
    $callbacks = $this->originalShutdownCallbacks;

    // Return the user to the original one.
    $user = $this->originalUser;
    drupal_save_session(TRUE);

    // Ensure that internal logged in variable and cURL options are reset.
    $this->loggedInUser = FALSE;
    $this->additionalCurlOptions = array();

    // Reload module list and implementations to ensure that test module hooks
    // aren't called after tests.
    module_list(TRUE);
    module_implements('', FALSE, TRUE);

    // Reset the Field API.
    field_cache_clear();

    // Rebuild caches.
    $this->refreshVariables();

    // Reset language.
    $language = $this->originalLanguage;
    if ($this->originalLanguageDefault) {
      $GLOBALS['conf']['language_default'] = $this->originalLanguageDefault;
    }

    // Close the CURL handler.
    $this->curlClose();
  }

  /**
   * Spec API methods
   * Methods implemented for performance, convenience or other good reasons.
   */

  /**
   * Log in user
   *
   * @param $username
   */
  public function login($username) {
    // Log out a previously logged in user
    if ($this->loggedInUser) {
      $this->logout();
    }

    // This will become the new logged in user
    // Overwrites the currently logged in admin user who runs the test
    // We'll need to return to this user after we've finished all tests
    // @see restoreOriginalSession()
    global $user;

    $user = user_load_by_name($username);
    if (!$user) {
      $this->fail(t('Login failed. User "%name" not found.', array('%name' => $username)));
      return;
    }

    $form_state = array('uid' => $user->uid);
    user_login_finalize($form_state);

    $loggedIn = is_numeric($user->login);
    $this->assertTrue($loggedIn, t('User %name logged in.', array('%name' => $username)));

    if ($loggedIn) {
      $this->loggedInUser = $user;
    }
  }

  /**
   * Log out current user
   */
  public function logout() {
    if (!$this->loggedInUser) {
      return;
    }

    module_invoke_all('user_logout', $this->loggedInUser);
    drupal_session_destroy_uid($this->loggedInUser->uid);
    $this->loggedInUser = FALSE;
  }

  /**
   * Call a menu path
   * Replacement for drupalGet()
   *
   * @path String the menu path
   * @deliver Boolean wether to deliver the full rendered page
   * @return HTML returned by menu callback
   */
  public function get($path, $deliver = TRUE) {
    // Get return value of menu path callback without delivering the whole page
    ob_start();
    menu_execute_active_handler($path, $deliver);
    $output = ob_get_contents();
    ob_end_clean();

    return $output;
  }

  /**
   * Load a node by title
   *
   * @param string $title
   *   The node title
   *
   * @return object $node
   *   The node object found or FALSE.
   */
  public function loadNodeByTitle($title) {
    $query = new EntityFieldQuery();
    $entities = $query->entityCondition('entity_type', 'node')
      ->propertyCondition('title', $title)
      ->range(0,1)
      ->execute();

    if (empty($entities['node'])) {
      return FALSE;
    }

    return node_load(array_shift(array_keys($entities['node'])));
  }
}
