<?php

/**
 * @file
 * Drush integration for the Spec module.
 *
 * The core test runner code is a fork of simpletest's test runner.
 * @see test.drush.inc
 */

/**
 * Implements hook_drush_command().
 */
function spec_drush_command() {
  $items['spec-run'] = array(
    'description' => dt('Run specs.'),
    'arguments' => array(
      'targets' => 'A test class, a test group. If omitted, a list of test classes and test groups is presented. Delimit multiple targets using commas.',
    ),
    'examples' => array(
      'drush spec-run, drush sr' => 'List all available spec classes and groups.',
      'drush sudo -u apache sr --all' => 'Run all available specs. Avoid permission related failures by running as web server user.',
      'drush sr SpecExampleTestCase' => 'Run a spec class.',
      'drush sr AppComponent' => 'Run all classes in a AppComponent group.',
      'drush sr AppComponent,Filter' => 'Run all specs from multiple groups/classes.',
      'drush sr SpecExampleTestCase --scenarios="testRestrictingAccessToAdministerPermissions, testRestrictingAccessToAdministerPermissionsOutline"' => 'Run particular scenarios in the specified class or group.',
    ),
    'options' => array(
      'all' => 'Run all available tests',
      'scenarios' => 'A comma delimited list of methods that should be run within the test class. Defaults to all methods.',
      'verbose' => 'Get live logs during the test run.',
      'dirty' => 'No cleaning up after tests have run.',
      'short' => 'Show only test summaries.',
    ),
    'aliases' => array('sr'),
  );
  return $items;
}

/**
 * Test-run command callback.
 *
 * @specs
 *   A comman delimited string of test classes or group names.
 */
function drush_spec_run($specs = NULL) {
  // @todo investigate if we really, really need this.
  //   Rebuilding cache is a performance killer.
  // if (drush_drupal_major_version() > 7) {
  //   cache()->delete('simpletest');
  // }
  // else {
  //   cache_clear_all('simpletest', 'cache');
  // }

  // Retrieve all tests and groups.
  list($groups, $all_tests) = drush_test_get_all_tests();

  if (drush_get_option('all')) {
    // Run all specs.
    foreach (array_keys($groups) as $group) {
      foreach (array_keys($groups[$group]) as $class) {
        if (_has_spec($class)) {
          drush_invoke_process('@self', 'spec-run', array($class));
        }
      }
    }
    return;
  }
  elseif (empty($specs)) {
    // No argument so list all groups/classes.
    return drush_spec_list($groups);
  }
  elseif (strpos($specs, ',') === FALSE && in_array($specs, $all_tests)) {
    // We got one valid class. Good, now run it.
    if (drush_get_option('short')) {
      _spec_run_feature_quietly($specs);
    }
    else {
      // Normal output
      _spec_run_feature($specs);
    }

    if (!drush_get_option('dirty')) {
      simpletest_clean_environment();
    }
    return;
  }

  // See if we got a list of specs.
  foreach (explode(',', $specs) as $spec) {
    $spec = trim($spec);
    if (in_array($spec, $all_tests)) {
      // Specific test class specified.
      drush_spec_run_class($spec);
    }
    if (isset($groups[$spec])) {
      // Specific group specified.
      foreach (array_keys($groups[$spec]) as $class) {
        drush_spec_run_class($class);
      }
    }
  }
}

/*
 * Run a test class via drush_invoke_process()
 * @param string $class
 *   The test class to run.
 *
 * @return
 *   If the command could not be completed successfully, FALSE.
 *   If the command was completed, this will return an associative
 *   array containing the results of the API call.
 *   @see drush_backend_get_result()
 */
function drush_spec_run_class($class) {
  $backend_options = array('output-label' => "$class ", 'integrate' => TRUE, 'output' => TRUE);
  $return = drush_invoke_process('@self', 'spec-run', array($class), drush_redispatch_get_options(), $backend_options);
  return $return;
}

/**
 * Run feature specs
 */
function _spec_run_feature($feature) {
  $class = _spec_to_camel_case($feature);
  $class = (strpos($class, 'TestCase') === FALSE) ? $class . 'TestCase' : $class;

  if (!class_exists($class)) {
    drush_log(t("The feature @class spec doesn't exist.", array('@class' => $class)), 'error');
    return;
  }

  // Enable realtime logging of events (defaults to OFF)
  $GLOBALS['realtime_logging'] = drush_get_option('verbose', FALSE);

  $test = new $class();
  $info = $test->getInfo();

  $scenarios = array();
  if ($scenarios_string = drush_get_option('scenarios')) {
    foreach (explode(',', $scenarios_string) as $scenario) {
      $scenarios[] = trim($scenario);
    }
  }

  drush_print(dt('Feature: '. $info['name']));
  drush_print(dt(_spec_indent(1) . $info['description']));

  $spec_log = spec_run($class, $scenarios);

  // Scenarios
  foreach ($spec_log as $scenario_method => $scenario) {
    $scenario_string = _scenario_string($scenario_method);

    if ($scenario['status'] == 'fail') {
      drush_log("\n\033[1;31m". _spec_indent(1) . "Scenario: ". $scenario_string ."\033[0m", 'failed');
    }
    else {
      drush_log("\n\033[1;32m". _spec_indent(1) . "Scenario: ". $scenario_string ."\033[0m", 'success');
    }

    // Scenario Examples
    foreach ($scenario['examples'] as $number => $example) {

      // Steps
      foreach ($example['steps'] as $step_text => $step) {
        if ($step['status'] == 'fail') {
          drush_log("\033[31m". _spec_indent(2) . $step_text ."\033[0m", 'failed');
        }
        else {
          drush_log("\033[32m". _spec_indent(2) . $step_text ."\033[0m", 'success');
        }

        // Scripts
        foreach ($step['scripts'] as $sript_method => $script) {
          if ($step['status'] == 'fail') {
            drush_log("\033[31m". _spec_indent(3) . dt($sript_method) ."\033[0m", 'failed');
          }
          else {
            // drush_print(_spec_indent(3) . $sript_method);
            drush_log("\033[32m". _spec_indent(3) . dt($sript_method) ."\033[0m", 'success');
          }

          // Logs
          foreach ($script['logs'] as $i => $log) {
            $message = _spec_filter_log_message(dt($log['message']));
            if ($log['status'] == 'fail') {
              drush_log("\033[31m". _spec_indent(4) . $message ."\033[0m", 'failed');
            }
            else {
              // @todo fix loss of colored output for multiline messages
              // @see http://api.aegirproject.org/api/Drush/includes--drush.inc/function/_drush_print_log/7.x-4.x
              //
              // 1) PROBLEM
              // _drush_print_log() breaks up the string after 80 characters and
              // thus the color information gets apparently lost.
              //
              // 2) FIX:
              //    - replace the print callback registered in drush_log() with custom callback spec_log()
              //      @see http://api.aegirproject.org/api/Drush/includes--drush.inc/function/drush_log/7.x-4.x
              //      $callback = drush_get_context('DRUSH_LOG_CALLBACK', '_drush_print_log');
              //      => how to register DRUSH_LOG_CALLBACK?
              //
              //    - copy code from _drush_print_log() and replace last part where
              //    message gets broken up
              // 3) TEST THIS FIX
              //
              //    => hack drush_log() in drush lib: remove breaking up of messages after 80 chars
              //       and check if multiline messages stay colored
              //

              drush_log("\033[32m". _spec_indent(4) . $message ."\033[0m", 'success');
            }
          }
        }
      }

      // New line after each scenario example
      drush_print();
    }
  }
}

/**
 * Run feature specs quietly.
 */
function _spec_run_feature_quietly($feature) {
  $class = _spec_to_camel_case($feature);
  $class = (strpos($class, 'TestCase') === FALSE) ? $class . 'TestCase' : $class;

  if (!class_exists($class)) {
    drush_log(t("The feature @class spec doesn't exist.", array('@class' => $class)), 'error');
    return;
  }

  // No realtime logging during quiet mode possible.
  $GLOBALS['realtime_logging'] = FALSE;

  $test = new $class();
  $info = $test->getInfo();

  $scenarios = array();
  if ($scenarios_string = drush_get_option('scenarios')) {
    foreach (explode(',', $scenarios_string) as $scenario) {
      $scenarios[] = trim($scenario);
    }
  }

  drush_print(dt($info['name']));
  $spec_log = spec_run($class, $scenarios);

  // Scenarios
  $scenarios = array('pass' => 0, 'fail' => 0);
  foreach ($spec_log as $scenario_method => $scenario) {
    $scenarios[$scenario['status']]++;
  }

  $msg = dt($scenarios['pass'] . " scenarios ok, " . $scenarios['fail'] . " scenarios failed.");
  if ($scenarios['fail'] == 0) {
    drush_log("\033[32m". $msg ."\033[0m", 'success');
  }
  else {
    drush_log("\033[31m". $msg ."\033[0m", 'failed');
  }

  // New line.
  drush_print();
}

/**
 * Print a listing of all available tests
 */
function drush_spec_list($groups) {
  $rows[] = array(dt('Spec'), dt('Description'));
  $rows[] = array('-------', '-----------');
  foreach ($groups as $group_name => $group_tests) {
    foreach ($group_tests as $test_class => $test_info) {
      if (!isset($test_info['spec']) || $test_info['spec'] == FALSE) {
        // This test has no specs.
        continue;
      }

      if (!isset($rows[$test_info['group']])) {
        $rows[$test_info['group']] = array($group_name, "");
      }
      $rows[] = array("  {$test_class}", "  {$test_info['name']}");
    }
  }
  return drush_print_table($rows, TRUE);
}

/**
 * Helper functions
 */

/**
 * Indent a log
 */
function _spec_indent($size) {
  $indent = "";
  for ($i=0; $i < $size; $i++) {
    $indent .= "  ";
  }

  return $indent;
}

/**
 * Generate scenario string from scenario method name
 */
function _scenario_string($method_name) {
  $string = preg_replace('/^test/', '', $method_name);
  return strtolower(trim(preg_replace('/(?<=\\w)(?=[A-Z])/'," $1", $string)));
}

/**
 * Check if a test class is flagged for Spec
 */
function _has_spec($class) {
  $test = new $class;
  $info = $test->getInfo();
  return (isset($info['spec']) && $info['spec']);
}
