<?php
/**
 * Callback: list all specs
 */
function spec_list() {
  $specs = spec_get_all();    
  return theme('spec_list', array('specs' => $specs));
}

/**
 * Callback: display a spec
 */
function spec_view($class_name) {
  $class = new $class_name;
  $feature = $class->getInfo();  
  $scenarios = spec_get_scenarios($class, $class_name);
    
  drupal_add_css(drupal_get_path('module', 'spec') .'/css/spec-view.css');
  drupal_set_title('Feature: '. $feature['name']);
  
  return theme('spec_view', array('feature' => $feature, 'scenarios' => $scenarios));
}

/**
 * Implements hook_theme().
 */
function spec_theme($existing, $type, $theme, $path) {
  return array(
    'spec_list' => array(
      'variables' => array('specs' => array()),
      'template' => 'templates/spec-list',
    ),

    'spec_view' => array(
      'variables' => array('feature' => NULL, 'scenarios' => array()),
      'template' => 'templates/spec-view',
    ),
  );
}

/**
 * Helper methods
 */

/**
 * Get a list of all specs provided by the system.
 *
 * @see simpletest_test_get_all()
 *
 * @return
 *   An array of specs keyed with the groups specified in each of the tests
 *   getInfo() method and then keyed by the test class. 
 *   This method looks only for tests flagged as "spec" inside getInfo(). 
 *
 *   An example of the array structure is provided below.
 *
 *   @code
 *     $groups['Blog'] => array(
 *       'BlogTestCase' => array(
 *         'name' => 'Blog functionality',
 *         'description' => 'Create, view, edit, delete, ...',
 *         'group' => 'Blog',
 *       ),
 *     );
 *   @endcode
 * @see simpletest_registry_files_alter()
 */
function spec_get_all() {
  $groups = &drupal_static(__FUNCTION__);

  if (!$groups) {
    // Load test information from cache if available, otherwise retrieve the
    // information from each tests getInfo() method.
    if ($cache = cache_get('spec', 'cache')) {
      $groups = $cache->data;
    }
    else {
      
      // Select all clases in files ending with .test.
      $classes = db_query("SELECT name FROM {registry} WHERE type = :type AND filename LIKE :name", array(':type' => 'class', ':name' => '%.test'))->fetchCol();

      // Check that each class has a getInfo() method and store the information
      // in an array keyed with the group specified in the test information.
      $groups = array();
      foreach ($classes as $class) {
        // Test classes need to implement getInfo() to be valid.
        if (class_exists($class) && method_exists($class, 'getInfo')) {
          $info = call_user_func(array($class, 'getInfo'));
          
          // Look for spec tests only
          if (!isset($info['spec']) || $info['spec'] != TRUE) {
            continue;
          }

          // If this test class requires a non-existing module, skip it.
          if (!empty($info['dependencies'])) {
            foreach ($info['dependencies'] as $module) {
              if (!drupal_get_filename('module', $module)) {
                continue 2;
              }
            }
          }

          $groups[$info['group']][$class] = $info;
        }
      }

      // Sort the groups and tests within the groups by name.
      uksort($groups, 'strnatcasecmp');
      foreach ($groups as $group => &$tests) {
        uksort($tests, 'strnatcasecmp');
      }

      // Allow modules extending core tests to disable originals.
      // TODO does this make sense here?
      // drupal_alter('simpletest', $groups);
      
      cache_set('spec', $groups);
      
    }
  }
  return $groups;
}

/**
 * Get all scenarios of a feature test class
 *
 * @param $class Object
 * @param $class_name String
 * @return Array scenarios with steps
 */
function spec_get_scenarios($class, $class_name) {
  $class_methods = get_class_methods(get_class($class));  
  $scenarios = array();
  foreach ($class_methods as $method) {
    // If the current method doesn't starts with "test", it's not a scenario.
    if (strtolower(substr($method, 0, 4)) != 'test') {
      continue;
    }
    
    // Convert method name into human readable scenario text
    $scenario_string = preg_replace('/^test/', '', $method);
    $scenario_string = strtolower(trim(preg_replace('/(?<=\\w)(?=[A-Z])/'," $1", $scenario_string)));
    
    $scenarios[$method] = array(
      'text' => $scenario_string,
      'steps' => SpecExampleTestCase::getSteps($class_name, $method),
    );
  }

  return $scenarios;
}
